@@ -1,4 +1,11 @@
-#ifndef RAFT_H
+#if defined(USE_SYSTEM_RAFT)
+
+#include <raft.h>
+#include <raft/uv.h>
+#include <raft/fixture.h>
+
+#elif !defined(RAFT_H)
+
 #define RAFT_H
 
 #include <stdarg.h>
@@ -9,6 +16,8 @@
 
 #include <uv.h>
 
+#include "lib/queue.h"
+
 #ifndef RAFT_API
 #define RAFT_API __attribute__((visibility("default")))
 #endif
@@ -188,6 +197,28 @@ enum {
 	RAFT_CHANGE       /* Raft configuration change. */
 };
 
+/**
+ * A small fixed-size inline buffer that stores extra data for a raft_entry
+ * that is different for each node in the cluster.
+ *
+ * A leader initializes the local data for an entry before passing it into
+ * raft_apply. This local data is stored in the volatile raft log and also
+ * in the persistent raft log on the leader. AppendEntries messages sent by
+ * the leader never contain the local data for entries.
+ *
+ * When a follower accepts an AppendEntries request, it invokes a callback
+ * provided by the FSM to fill out the local data for each new entry before
+ * appending the entries to its log (volatile and persistent). This local
+ * data doesn't have to be the same as the local data that the leader computed.
+ *
+ * When starting up, a raft node reads the local data for each entry for its
+ * persistent log as part of populating the volatile log.
+ */
+struct raft_entry_local_data {
+	/* Must be the only member of this struct. */
+	uint8_t buf[16];
+};
+
 /**
  * A single entry in the raft log.
  *
@@ -211,12 +242,27 @@ enum {
  *
  * This arrangement makes it possible to minimize the amount of memory-copying
  * when performing I/O.
+ *
+ * The @is_local field is set to `true` by a leader that appends an entry to its
+ * volatile log. It is set to `false` by a follower that copies an entry received
+ * via AppendEntries to its volatile log. It is not represented in the AppendEntries
+ * message or in the persistent log. This field can be used by the FSM's `apply`
+ * callback to handle a COMMAND entry differently depending on whether it
+ * originated locally.
+ *
+ * Note: The @local_data and @is_local fields do not exist when we use an external
+ * libraft, because the last separate release of libraft predates their addition.
+ * The ifdef at the very top of this file ensures that we use the system raft headers
+ * when we build against an external libraft, so there will be no ABI mismatch as
+ * a result of incompatible struct layouts.
  */
 struct raft_entry
 {
-	raft_term term;      /* Term in which the entry was created. */
-	unsigned short type; /* Type (FSM command, barrier, config change). */
+	raft_term term;         /* Term in which the entry was created. */
+	unsigned short type;    /* Type (FSM command, barrier, config change). */
+	bool is_local;          /* Placed here so it goes in the padding after @type. */
 	struct raft_buffer buf; /* Entry data. */
+	struct raft_entry_local_data local_data;
 	void *batch;            /* Batch that buf's memory points to, if any. */
 };
 
@@ -802,7 +848,7 @@ struct raft
 			raft_index
 			    round_index; /* Target of the current round. */
 			raft_time round_start; /* Start of current round. */
-			void *requests[2]; /* Outstanding client requests. */
+			queue requests; /* Outstanding client requests. */
 			uint32_t
 			    voter_contacts; /* Current number of voting nodes we
 					       are in contact with */
@@ -1029,7 +1075,7 @@ RAFT_API int raft_voter_contacts(struct raft *r);
 	void *data;            \
 	int type;              \
 	raft_index index;      \
-	void *queue[2];        \
+	queue queue;           \
 	uint8_t req_id[16];    \
 	uint8_t client_id[16]; \
 	uint8_t unique_id[16]; \
@@ -1073,6 +1119,7 @@ struct raft_apply
 RAFT_API int raft_apply(struct raft *r,
 			struct raft_apply *req,
 			const struct raft_buffer bufs[],
+			const struct raft_entry_local_data local_data[],
 			const unsigned n,
 			raft_apply_cb cb);
 
